use 5.010001;
use strict;
use warnings;
use Type::Tiny::XS ();

package Type::Tiny::XS::Util;

our $AUTHORITY = 'cpan:TOBYINK';
our $VERSION   = '0.022';

1;

__END__

=pod

=encoding utf-8

=head1 NAME

Type::Tiny::XS::Util - support functions for Type::Tiny::XS

=head1 DESCRIPTION

This module provides no supported functions.

=head1 BUGS

Please report any bugs to
L<http://rt.cpan.org/Dist/Display.html?Queue=Type-Tiny-XS>.

=head1 SEE ALSO

L<Type::Tiny::XS>.

=head1 AUTHOR

Toby Inkster E<lt>tobyink@cpan.orgE<gt> forked all this from
L<Mouse::Util>.

=head1 COPYRIGHT AND LICENCE

This software is copyright (c) 2014 by Toby Inkster.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=head1 DISCLAIMER OF WARRANTIES

THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

